/*********************************************
 *
 * Beispiel: Analog/Digital Konverter
 *
 * Analogeingang: ADC4
 * Takt 12MHz
 *
 *********************************************/
#include <stdio.h>
#include <avr/io.h>
#ifndef F_CPU
#define F_CPU 12000000UL     // Quarz 12MHz
#endif
#include <util/delay.h>
#include "lcd.h"

uint16_t ReadChannel(uint8_t mux)
{
    uint8_t i;
    uint16_t result;
 
    ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1);    // Frequenzvorteiler 
                                                     // setzen auf 64 und ADC aktivieren
 
    ADMUX = mux;                      // Kanal waehlen
    ADMUX |= (1<<REFS0);              // AVcc als Referenzspannung nutzen 
 
    /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
       also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
    ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
    while ( ADCSRA & (1<<ADSC) ) {
       ;     // auf Abschluss der Konvertierung warten 
    }
    result = ADCW;  // ADCW muss einmal gelesen werden,
                    // sonst wird Ergebnis der nächsten Wandlung
                    // nicht übernommen.
 
    /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
    result = 0; 
    for( i=0; i<4; i++ )
    {
      ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
      while ( ADCSRA & (1<<ADSC) ) {
        ;                             // auf Abschluss der Konvertierung warten
      }
      result += ADCW;		      // Wandlungsergebnisse aufaddieren
    }
    ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
 
    result /= 4;                      // Summe durch vier teilen = arithm. Mittelwert
 
    return result;
}

void long_delay(uint16_t ms) 
{
    for (; ms>0; ms--) _delay_ms(1);
}


int main( void )
{
    char buffer[16];
    uint16_t adcval;

    DDRB |= (1<<2);
    PORTB &= ~ (1<<2);                  // Kontrastsignal

    DDRB |= (1<<1);
    PORTB |= (1<<1);                  // Hintergrundbeleuchtung

    lcd_init(LCD_DISP_ON);              // LCD initialisieren
    lcd_clrscr(); 

    while( 1 ) {
       adcval = ReadChannel(4);         // von ADC4 lesen
       sprintf (buffer, "ADC: %d", adcval);
       lcd_clrscr(); 
       lcd_puts(buffer);
       long_delay(200);
  }

    return 0;
}

