/*********************************************
 *
 * Beispiel: 1 Sekunde - Timer 
 *
 * Takt 12MHz
 *
 *********************************************/

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"

#define F_CPU 12000000UL                // Quarz 12MHz
#define DEBOUNCE 256UL                  // debounce clock (256Hz = 4msec)
 
uint8_t prescaler;
uint8_t volatile second;		// count seconds
uint8_t volatile minute;		// count minute
uint8_t volatile hour;			// count hour
uint8_t volatile DD;			// count 
uint8_t volatile MM;			// count 
uint16_t volatile YY;			// count 
uint8_t volatile WD;			// count Weekday 1=Mo
uint8_t volatile tmp_mm;		// tmp minute
uint8_t volatile tmp_hh;		// tmp hour
uint8_t volatile tmp_DD;		// tmp
uint8_t volatile tmp_MM;		// tmp
uint16_t volatile tmp_YY;		// tmp
uint8_t volatile tmp_WD;		// tmp
uint8_t state; 
uint8_t last_state = 0; 
uint16_t counter;
uint8_t rx_count; 

 
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
 
#if F_CPU % DEBOUNCE                    // bei rest
  OCR1A = F_CPU / DEBOUNCE - 1;		// compare DEBOUNCE - 1 times
#endif
  
  if (PINC & (1 << PC4)) {
     PORTD |= (1 << PD6);
     state = 1;
  } else {
     PORTD &= ~(1 << PD6);
     state = 0;
  }

  if ( state == 1 && last_state == 0 ) {
    // begin flanke, begin second
//    sprintf (buffer, "XX: %d", counter);
//    lcd_clrscr();
//    lcd_puts(buffer);
    if ( counter > 255 ) {
      // sec 0
      second = 0;
      rx_count = -1;
      minute = tmp_mm;
      hour = tmp_hh;
      DD = tmp_DD;
      MM = tmp_MM;
      YY = tmp_YY;
      WD = tmp_WD;
      tmp_mm = 0;
      tmp_hh = 0;
      tmp_DD = 0;
      tmp_MM = 0;
      tmp_YY = 2000;
      tmp_WD = 0;
      prescaler = 0;
    }

    if ( counter < 222 ) {
      // log 1
//      if ( second = 20 ) {
//        // startbit o.k.
//      }
        if ( rx_count == 21 ) { tmp_mm = 1; }
        if ( rx_count == 22 ) { tmp_mm += 2; } 
        if ( rx_count == 23 ) { tmp_mm += 4; }
        if ( rx_count == 24 ) { tmp_mm += 8; }
        if ( rx_count == 25 ) { tmp_mm += 10; }
        if ( rx_count == 26 ) { tmp_mm += 20; }
        if ( rx_count == 27 ) { tmp_mm += 40; }

        if ( rx_count == 29 ) { tmp_hh = 1; }
        if ( rx_count == 30 ) { tmp_hh += 2; }
        if ( rx_count == 31 ) { tmp_hh += 4; }
        if ( rx_count == 32 ) { tmp_hh += 8; }
        if ( rx_count == 33 ) { tmp_hh += 10; }
        if ( rx_count == 34 ) { tmp_hh += 20; }

        if ( rx_count == 36 ) { tmp_DD = 1; }
        if ( rx_count == 37 ) { tmp_DD += 2; }
        if ( rx_count == 38 ) { tmp_DD += 4; }
        if ( rx_count == 39 ) { tmp_DD += 8; }
        if ( rx_count == 40 ) { tmp_DD += 10; }
        if ( rx_count == 41 ) { tmp_DD += 20; }
      
        if ( rx_count == 42 ) { tmp_WD = 1; }
        if ( rx_count == 43 ) { tmp_WD += 2; }
        if ( rx_count == 44 ) { tmp_WD += 4; }

        if ( rx_count == 45 ) { tmp_MM = 1; }
        if ( rx_count == 46 ) { tmp_MM += 2; }
        if ( rx_count == 47 ) { tmp_MM += 4; }
        if ( rx_count == 48 ) { tmp_MM += 8; }
        if ( rx_count == 49 ) { tmp_MM += 10; }

        if ( rx_count == 50 ) { tmp_YY = 1; }
        if ( rx_count == 51 ) { tmp_YY += 2; }
        if ( rx_count == 52 ) { tmp_YY += 4; }
        if ( rx_count == 53 ) { tmp_YY += 8; }
        if ( rx_count == 54 ) { tmp_YY += 10; }
        if ( rx_count == 55 ) { tmp_YY += 20; }
        if ( rx_count == 56 ) { tmp_YY += 40; }
        if ( rx_count == 57 ) { tmp_YY += 80; }

    }


    counter=0;
    rx_count++;
  }

  if ( state == 0 && last_state == 1 ) {
    // end flanke

  }

  if ( state == 0 ) counter++;

  last_state = state;

  if( --prescaler == 0 ){
    prescaler = (uint8_t)DEBOUNCE;
    second++;				// exact one second over
//    if (second == 60) {
//      second = 0;
//      minute++;
//      if (minute == 60) {
//        minute = 0;
//        hour++;
//        if (hour == 24) {
//          hour = 0;
//        }
//      }
//    }
#if F_CPU % DEBOUNCE			// handle remainder
    OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1; // compare once per second
#endif
  }
}

int main( void )
{
    char buffer[16];
    uint8_t last_second=61;
    const char day[7][3] = {"Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"};

    DDRC &= ~( 1 << PC4 );      // PC5 an PORTC als Eingang setzen
    PORTC |= ( 1 << PC4 );      // Pullup-Widerstand aktivieren
    DDRD = ( 1 << PD6 );        // PD6 an PORTD als Ausgang setzen

    TCCR1B = 1 << WGM12 ^ 1 << CS10;    // divide by 1
					// clear on compare
    OCR1A = F_CPU / DEBOUNCE - 1;       // Output Compare Register
    TCNT1 = 0;                          // Timmer startet mit 0
    second = 0;
    prescaler = (uint8_t)DEBOUNCE;      //software teiler
 
    TIMSK = 1 << OCIE1A;                // beim Vergleichswertes Compare Match                    
                                        // Interrupt (SIG_OUTPUT_COMPARE1A)
    sei();				// Interrupts aktivieren

    DDRB |= (1<<2);
    PORTB &= ~ (1<<2);                  // Kontrastsignal

    DDRB |= (1<<1);
    PORTB |= (1<<1);                  // Backlight

    lcd_init(LCD_DISP_ON);              // LCD initialisieren
    lcd_clrscr(); 

    while( 1 ) {
        if (last_second != second) {
          sprintf (buffer, "    %2d:%02d:%02d", hour, minute, second);
          lcd_clrscr(); 
          lcd_puts(buffer);
//          lcd_gotoxy(0,1);
//          sprintf (buffer, "%i %i%i%i%i%i%i%i%i%i%i%i",is, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10);
//          lcd_puts(buffer);
          lcd_gotoxy(0,1);
          sprintf (buffer, " %s %02d.%02d.%04d", &day[WD-1][0], DD, MM, YY);
          lcd_puts(buffer);
          last_second = second;
        }
  }

    return 0;
}

