/*********************************************
 *
 * Beispiel: LCD Demo
 *
 * LCD-Konfiguration: siehe lcd.h
 * Kontrast: PWM an OC1B
 * Beleuchtung: PWM an OC1A
 * Takt 12MHz
 *
 *********************************************/
#include <stdlib.h>
#include <avr/io.h>
#ifndef F_CPU
#define F_CPU 12000000UL     // Quarz 12MHz
#endif
#include <util/delay.h>
#include "lcd.h"

static const PROGMEM unsigned char copyRightChar[] =
{
        0x07, 0x08, 0x13, 0x14, 0x14, 0x13, 0x08, 0x07,
        0x00, 0x10, 0x08, 0x08, 0x08, 0x08, 0x10, 0x00
};

/*   Zeichendarstellung copyRightChar

     ..### .....
     .#... #....
     #..## .#...
     #.#.. .#...
     #.#.. .#...
     #..## .#...
     .#... #....
     ..### .....
 */


/* PWM Funktionen */
void pwm_init(void)
{
  DDRB |= (1<<1) | (1<<2); /* PWM0/1 are output */

  // 8 bit PWM on OC1A and OC1B (inverted)
  TCCR1A = (1 << WGM10) | (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
  // timer 1 running on CPU clock/8
  TCCR1B = (1 << CS11);

  OCR1B = 0xd2;        // initiale Vergleichswerte
  OCR1A = 0xff;
}

void set_contrast(int value)
{
  OCR1B = value;       // lower voltage is higher contrast
                       // note: OC1B runs inverted
}

void set_brightness(int value)
{
  OCR1A = value;      // higher voltage is higher brightness
}


void long_delay(uint16_t ms) 
{
    for (; ms>0; ms--) _delay_ms(1);
}


int main( void )
{
    char buffer[7];
    uint8_t  num=134;
    uint8_t i;

    pwm_init();                         // PWM initialisieren

    lcd_init(LCD_DISP_ON);              // LCD initialisieren
    lcd_clrscr(); 

    lcd_puts("LCD Demo ...");

    while( 1 ) {
        /* dimm Beleuchtung */
        for (i=1; i<254; i++)
        {
           set_brightness(i);
           long_delay(15);
        } 

        /*
         * Test 1:  write text to display
         */

        /* clear display and home cursor */
        lcd_clrscr();

        /* put string to display (line 1) with linefeed */
        lcd_puts("LCD Test Line 1\n");

        /* cursor is now on second line, write second line */
        lcd_puts("Line 2");

        /* move cursor to position 8 on line 2 */
        lcd_gotoxy(7,1);

        /* write single char to display */
        lcd_putc(':');

        /* wait ... */
        long_delay(3000);

        /*
         * Test 2: use lcd_command() to turn on cursor
         */

        /* turn on cursor */
        lcd_command(LCD_DISP_ON_CURSOR);

        /* put string */
        lcd_puts( "CurOn");

        /* wait ... */
        long_delay(3000);


        /*
         * Test 3: display shift
         */

        lcd_clrscr();     /* clear display home cursor */

        /* put string from program memory to display */
        lcd_puts_P( "Line 1 longer than 14 characters\n" );
        lcd_puts_P( "Line 2 longer than 14 characters" );

        /* move BOTH lines one position to the left */
        lcd_command(LCD_MOVE_DISP_LEFT);

        /* wait ... */
        long_delay(3000);

        /* turn off cursor */
        lcd_command(LCD_DISP_ON);


        /*
         *   Test: Display integer values
         */

        lcd_clrscr();   /* clear display home cursor */

        /* convert interger into string */
        itoa( num , buffer, 10);

        /* put converted string to display */
        lcd_puts(buffer);

        /* wait ... */
        long_delay(3000);


        /*
         *  Test: Display userdefined characters
         */

        lcd_clrscr();   /* clear display home cursor */

        lcd_puts("Copyright: ");

        /*
         * load two userdefined characters from program memory
         * into LCD controller CG RAM location 0 and 1
         */
        lcd_command(1 << LCD_CGRAM);  /* set CG RAM start address 0 */
        for(i=0; i<16; i++)
        {
            lcd_data(pgm_read_byte_near(&copyRightChar[i]));
        }

        /* move cursor to position 0 on line 2 */
        /* Note: this switched back to DD RAM adresses */
        lcd_gotoxy(0,1);

        /* display user defined (c), built using two user defined chars */
        lcd_putc(0);
        lcd_putc(1);

        /* wait ... */
        long_delay(3000);

        lcd_clrscr();     /* clear display home cursor */
        /* dimm Beleuchtung */
        for (i=254; i>0; i--)
        {
           set_brightness(i);
           long_delay(15);
        } 

        /* wait ... */
        long_delay(3000);

    }

    return 0;
}

