/*********************************************
 *
 * Beispiel: 1 Sekunde - Timer 
 *
 * Takt 12MHz
 *
 *********************************************/

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"

#define F_CPU 12000000UL                // Quarz 12MHz
#define DEBOUNCE 256UL                  // debounce clock (256Hz = 4msec)
 
uint8_t prescaler;
uint8_t volatile second;		// count seconds
uint8_t volatile minute;		// count minute
uint8_t volatile hour;			// count hour
 
 
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
 
#if F_CPU % DEBOUNCE                    // bei rest
  OCR1A = F_CPU / DEBOUNCE - 1;		// compare DEBOUNCE - 1 times
#endif
  if( --prescaler == 0 ){
    prescaler = (uint8_t)DEBOUNCE;
    second++;				// exact one second over
    if (second == 60) {
      second = 0;
      minute++;
      if (minute == 60) {
        minute = 0;
        hour++;
        if (hour == 24) {
          hour = 0;
        }
      }
    }
#if F_CPU % DEBOUNCE			// handle remainder
    OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1; // compare once per second
#endif
  }
}


int main( void )
{
    char buffer[16];
    uint8_t last_second=61;

    TCCR1B = 1 << WGM12 ^ 1 << CS10;    // divide by 1
					// clear on compare
    OCR1A = F_CPU / DEBOUNCE - 1;       // Output Compare Register
    TCNT1 = 0;                          // Timmer startet mit 0
    second = 0;
    prescaler = (uint8_t)DEBOUNCE;      //software teiler
 
    TIMSK = 1 << OCIE1A;                // beim Vergleichswertes Compare Match                    
                                        // Interrupt (SIG_OUTPUT_COMPARE1A)
    sei();				// Interrupts aktivieren

    DDRB |= (1<<2);
    PORTB &= ~ (1<<2);                  // Kontrastsignal

    lcd_init(LCD_DISP_ON);              // LCD initialisieren
    lcd_clrscr(); 

    while( 1 ) {
        if (last_second != second) {
          sprintf (buffer, "Time: %2d:%02d:%02d", hour, minute, second);
          lcd_clrscr(); 
          lcd_puts(buffer);
          last_second = second;
        }
  }

    return 0;
}

