/*********************************************
 *
 * Beispiel: Sekundentakt mit 8bit-Timer (Timer2)
 *
 * Takt 12MHz
 * Vorteiler 256
 *
 *       (OCR2 + 1) * PRESCALER
 *  t = ------------------------
 *               F_CPU
 *
 *********************************************/

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"

#define F_CPU 12000000UL                // Quarz 12MHz
#define PRESCALER 256UL                 // timer prescaler
#define DEBOUNCE  125
 
uint16_t counter;
uint8_t volatile second;		// count seconds
uint8_t volatile minute;		// count minute
uint8_t volatile hour;			// count hour
 
/* Timer Interrupt service routine */ 
SIGNAL (SIG_OUTPUT_COMPARE2)
{
  if( --counter == 0 ){
    counter = (uint16_t)(F_CPU/PRESCALER/DEBOUNCE);
    second++;				// one second over
    if (second == 60) {
      second = 0;
      minute++;
      if (minute == 60) {
        minute = 0;
        hour++;
        if (hour == 24) {
          hour = 0;
        }
      }
    }
  }
}

/* PWM Funktionen */
void pwm_init(void)
{
  DDRB |= (1<<1) | (1<<2); /* PWM0/1 are output */

  // 8 bit PWM on OC1A and OC1B (inverted)
  TCCR1A = (1 << WGM10) | (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
  // timer 1 running on CPU clock/8
  TCCR1B = (1 << CS11);

  OCR1B = 0xd2;        // initiale Vergleichswerte
  OCR1A = 0xff;
}

int main( void )
{
    char buffer[16];
    uint8_t last_second=61;

    TCCR2 = 1 << WGM21 ^ 1 << CS21 ^ 1 << CS22;
                                        // Timer2, WGM21 = CTC-Mode
                                        // CS22, CS21 prescaler = 256
    OCR2 = (uint8_t)DEBOUNCE -1;        // Output Compare Register
    TIMSK = 1 << OCIE2;                 // Timer2 beim Vergleichswertes Compare Match

    second = 0;
    counter = (uint16_t)(F_CPU/PRESCALER/DEBOUNCE); //software teiler

    pwm_init();                         // PWM initialisieren
    sei();				// Interrupts aktivieren

    lcd_init(LCD_DISP_ON);              // LCD initialisieren
    lcd_clrscr(); 

    while( 1 ) {
        if (last_second != second) {
          sprintf (buffer, "Time: %2d:%02d:%02d", hour, minute, second);
          lcd_clrscr(); 
          lcd_puts(buffer);
          last_second = second;
        }
  }

    return 0;
}

